import 'dart:convert';

import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/constant/value.dart';
import 'package:build/src/builder/build_step.dart';
import 'package:source_gen/source_gen.dart';
import 'package:singleton_annotation/singleton_annotation.dart';

class SingletonClassGenerator extends GeneratorForAnnotation<Singleton> {
  @override
  generateForAnnotatedElement(
      Element element, ConstantReader annotation, BuildStep buildStep) {
    if (element is! ClassElement) {
      throw UnsupportedError('`@Singleton` can only be used on classes.');
    }

    final constructorName = (annotation.peek('constructor') == null)
        ? ''
        : '.' + annotation.peek('constructor').stringValue;
    final classElement = element as ClassElement;
    final className = classElement.name;
    /*
     * TODO: Validate the constructor syntax, add argument
     * 
    final constructorElement = classElement.constructors?.singleWhere((elm) {
      print(elm.name);
      if (elm.name == constructorName) return true;
      return false;
    }, orElse: () {
      return null;
    });
    final needArgs = constructorElement?.parameters?.length == 1 &&
        constructorElement.parameters[0].isPositional;

    print('Need args? $needArgs');
    final isValidConstructor = constructorElement.parameters.length == 1 &&
        constructorElement.parameters[0].isPositional &&
        constructorElement.parameters[0].type.name == 'Map';
    */
    final functionName = '_\$$className';
    final singletonBuilder = '''
    ${element.name} _cache;
    ${element.name} ${functionName}() {
      _cache ??= $className$constructorName();
      return _cache;
    }
    ''';
    return singletonBuilder;
  }
}

/// From here is for the next version

dynamic _getRawValue(DartObject value) {
  return value.toBoolValue() ??
      value.toDoubleValue() ??
      value.toFunctionValue() ??
      value.toIntValue() ??
      value.toListValue() ??
      value.toMapValue() ??
      value.toSetValue() ??
      value.toStringValue() ??
      value.toSymbolValue() ??
      value.toTypeValue() ??
      value;
}

dynamic _getRawArgs(ConstantReader args) {
  final ret = args.boolValue ??
      args.doubleValue ??
      args.intValue ??
      args.listValue ??
      args.objectValue ??
      args.stringValue ??
      args.symbolValue ??
      args.typeValue;
  return ret;
}
